/*
 * [Grams.java]
 *
 * Summary: Display a number in English words used to measure in grams.
 *
 * Copyright: (c) 1999-2017 Roedy Green, Canadian Mind Products, http://mindprod.com
 *
 * Licence: This software may be copied and used freely for any purpose but military.
 *          http://mindprod.com/contact/nonmil.html
 *
 * Requires: JDK 1.8+
 *
 * Created with: JetBrains IntelliJ IDEA IDE http://www.jetbrains.com/idea/
 *
 * Version History:
 *  1.0 1999-02-05
 */
package com.mindprod.inwords;

/**
 * Display a number in English words used to measure in grams.
 * <p/>
 * e.g.
 * 12345 -&gt; &quot;twelve kilograms, forty-five grams&quot;
 * <p/>
 * Why is this code written in an unusual way?   see inwords.use
 *
 * @author Roedy Green, Canadian Mind Products
 * @version 1.0 1999-02-05
 * @since 1999-02-05
 */
public final class Grams extends AmericanEnglish implements ToWords
    {
    private static final int FIRST_COPYRIGHT_YEAR = 1999;

    /**
     * undisplayed copyright notice
     *
     * @noinspection UnusedDeclaration
     */
    private static final String EMBEDDED_COPYRIGHT =
            "Copyright: (c) 1999-2017 Roedy Green, Canadian Mind Products, http://mindprod.com";

    private static final String GRAM = "gram";

    private static final String GRAMS = "grams";

    private static final String MINUS = "minus";

    private static final String ZERO = "zero";

    private static final String[] prefixName = {
            /*
            * American: unit, ten, hundred, thousand, million, billion, trillion,
            * quadrillion, quintillion
            */
            "", "kilo", "mega", "giga", "tera", "peta", "exa" };

    private static final int[] divisor = {
            /*
            * HowToProcess many of this group is needed to form one of the succeeding group.
            */
            /*
            * American: unit, thousand, million, billion, trillion, quadrillion,
            * quintillion
            */
            1000, 1000, 1000, 1000, 1000, 1000, 1000 };

    /**
     * test harness
     *
     * @param args not used
     */
    public static void main( String[] args )
        {
        Test.test( new Grams() );
        } // end main

    /**
     * convert long integer into American English words. e.g. 12345 -> "twelve kilograms, forty-five grams" Handles
     * negative and positive integers on range -Long.MAX_VALUE .. Long.MAX_VALUE; It cannot handle Long.MIN_VALUE;
     *
     * @param num number to convert to words
     *
     * @return words
     */
    @SuppressWarnings( { "WeakerAccess" } )
    public String toWords( long num )
        {
        if ( num == 0 )
            {
            return ZERO + " " + GRAMS;
            }
        boolean negative = ( num < 0 );
        if ( negative )
            {
            num = -num;
            }
        String s = "";
        // Work least significant digit to most, right to left.
        // until high order part is all 0s.
        for ( int group = 0; num > 0; group++ )
            {
            int remdr = ( int ) ( num % divisor[ group ] );
            num = num / divisor[ group ];
            // check for 1100 .. 1999, 2100..2999, ... 5200..5999
            // but not 1000..1099, 2000..2099, ...
            // Special case written as fifty-nine hundred.
            // e.g. thousands digit is 1..5 and hundreds digit is 1..9
            // Only when no further higher order.
            if ( 1 <= num && num <= 5 && remdr >= 100 )
                {
                remdr += num * 1000;
                num = 0;
                } // end if
            if ( remdr == 0 )
                {
                continue;
                }
            String t = super.toWords( remdr );
            boolean comma = ( s.length() != 0 );
            boolean plural = ( remdr != 1 );
            s = t
                + " "
                + prefixName[ group ]
                + ( plural ? GRAMS : GRAM )
                + ( comma ? "," : "" )
                + " "
                + s;
            } // end for
        s = s.trim();
        if ( negative )
            {
            s = MINUS + " " + s;
            }
        return s;
        } // end toWords
    } // end Grams
